/*************************************************************************
 *
 *  OpenOffice.org - a multi-platform office productivity suite
 *
 *  $RCSfile: expl.cxx,v $
 *
 *  $Revision: 1.3 $
 *
 *  last change: $Author: hr $ $Date: 2006-06-19 21:50:15 $
 *
 *  The Contents of this file are made available subject to
 *  the terms of GNU Lesser General Public License Version 2.1.
 *
 *
 *    GNU Lesser General Public License Version 2.1
 *    =============================================
 *    Copyright 2005 by Sun Microsystems, Inc.
 *    901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *    This library is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU Lesser General Public
 *    License version 2.1, as published by the Free Software Foundation.
 *
 *    This library is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    Lesser General Public License for more details.
 *
 *    You should have received a copy of the GNU Lesser General Public
 *    License along with this library; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *    MA  02111-1307  USA
 *
 ************************************************************************/
#include "expl.hxx"
#include "shapes.hxx"
#include "invader.hrc"

#ifndef _SV_OUTDEV_HXX //autogen
#include <vcl/outdev.hxx>
#endif

Explosion::Explosion(ResMgr* pRes) :
            ExplListe(0,1),
            pExpl1(0L),
            pExpl2(0L),
            pExpl3(0L)
{
    pExpl1 = ImplLoadImage( EXPLOS1, pRes );
    pExpl2 = ImplLoadImage( EXPLOS2, pRes );
    pExpl3 = ImplLoadImage( EXPLOS3, pRes );
}

Explosion::~Explosion()
{
    delete pExpl1;
    delete pExpl2;
    delete pExpl3;
}

void Explosion::Paint(OutputDevice& rDev)
{
    unsigned long i;
    for( i = 0; i < Count(); i++ )
    {
        switch(GetMode(i))
        {
            case EXPL1:
                rDev.DrawImage(GetPoint(i),*pExpl1);
                SetMode(i,EXPL2);
                break;
            case EXPL2:
                rDev.DrawImage(GetPoint(i),*pExpl2);
                SetMode(i,EXPL3);
                break;
            case EXPL3:
                rDev.DrawImage(GetPoint(i),*pExpl3);
                SetMode(i,EXPL4);
                break;
            case EXPL4:
                rDev.DrawImage(GetPoint(i),*pExpl2);
                SetMode(i,EXPL5);
                break;
            case EXPL5:
                rDev.DrawImage(GetPoint(i),*pExpl3);
                SetMode(i,EXPL6);
                break;
            case EXPL6:
                rDev.DrawImage(GetPoint(i),*pExpl2);
                SetMode(i,EXPL7);
                break;
            case EXPL7:
                rDev.DrawImage(GetPoint(i),*pExpl1);
                SetMode(i,EXPLNONE);
                break;
            case EXPLNONE:
                SetMode(i,EXPLDEL);
                break;
            case EXPL8:
                break;
            case EXPLDEL:
                break;
        }
    }

//  RemoveExpl();
}

BOOL Explosion::RemoveExpl()
{
    Expl_Impl* pWork;

    for(long i=Count()-1; i >= 0; i--)
    {
        if(GetMode(i) == EXPLDEL)
        {
            pWork = GetObject(i);
            Remove(pWork);
            delete pWork;
        }
    }

    if(Count())
        return FALSE;
    else
        return TRUE;
}

void Explosion::ClearAll()
{
    unsigned long i;
    for( i = 0;  i < Count(); i++ )
        delete GetObject(i);

    Clear();
}

void Explosion::InsertExpl(Point& rPoint)
{
    Expl_Impl* pWork = new Expl_Impl();

    pWork->aPos     = rPoint;
    pWork->eMode    = EXPL1;
    Insert(pWork);
}
